/*
Copyright 2010 James Pether Sörling Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the License. You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the specific language governing permissions and limitations under the License. 
	$Id
*/

package com.hack23.cia.functionalintegrationtest.scenarios.web.controller.common;

import org.hamcrest.Description;
import org.hamcrest.TypeSafeMatcher;
import org.jmock.Mockery;
import org.jmock.integration.junit4.JUnit4Mockery;
import org.junit.After;
import org.junit.Before;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.security.authentication.encoding.Md5PasswordEncoder;
import org.springframework.security.authentication.encoding.PasswordEncoder;
import org.springframework.test.context.transaction.AfterTransaction;
import org.springframework.test.context.transaction.BeforeTransaction;

import com.hack23.cia.functionalintegrationtest.scenarios.service.common.TestRequestFactory;
import com.hack23.cia.model.application.impl.common.UserSession;
import com.hack23.cia.model.application.impl.user.UserAccountActionEvent;
import com.hack23.cia.service.api.application.ApplicationManager;
import com.hack23.cia.service.api.application.ApplicationRequest;
import com.hack23.cia.service.api.user.LoginRequest;
import com.hack23.cia.testfoundation.AbstractFunctionalIntegrationTransactionalTest;
import com.hack23.cia.web.controller.application.FrontController;
import com.hack23.cia.web.controller.common.ActionHandler;
import com.hack23.cia.web.controller.common.UserStateService;
import com.hack23.cia.web.viewfactory.api.common.ModelAndView;
import com.hack23.cia.web.viewfactory.api.common.ViewFactoryService;

/**
 * The Class AbstractFunctionalActionWithoutViewFactoryTest.
 */
public abstract class AbstractFunctionalActionWithoutViewFactoryTest extends
        AbstractFunctionalIntegrationTransactionalTest {

    /**
     * The Class ModelAndViewMatcher.
     */
    public class ModelAndViewMatcher extends TypeSafeMatcher<ModelAndView> {

        /** The match class. */
        @SuppressWarnings("unchecked")
        private final Class matchClass;

        /** The model and view. */
        private ModelAndView modelAndView;

        /**
         * Instantiates a new model and view matcher.
         *
         * @param matchClass the match class
         */
        @SuppressWarnings("unchecked")
        public ModelAndViewMatcher(Class matchClass) {
            this.matchClass = matchClass;
        }

        /*
         * (non-Javadoc)
         * 
         * @see org.hamcrest.SelfDescribing#describeTo(org.hamcrest.Description)
         */
        @Override
        public void describeTo(Description arg0) {
            arg0.appendText("Wrong View : was "
                    + modelAndView.getClass().getSimpleName() + " , expected "
                    + matchClass.getSimpleName());
        }

        /**
         * Gets the model and view.
         *
         * @return the model and view
         */
        public final ModelAndView getModelAndView() {
            return this.modelAndView;
        }

        /*
         * (non-Javadoc)
         * 
         * @see org.hamcrest.TypeSafeMatcher#matchesSafely(java.lang.Object)
         */
        @Override
        public boolean matchesSafely(final ModelAndView modelAndView) {
            this.modelAndView = modelAndView;            
            return matchClass.getSimpleName().equals(
                    modelAndView.getClass().getSimpleName());
        }
    }

    /** The application manager. */
    @Autowired
    @Qualifier("applicationManager")
    protected ApplicationManager applicationManager;

    /** The context. */
    protected Mockery context = new JUnit4Mockery();

    /** The front controller. */
    @Autowired
    @Qualifier("controller")
    protected FrontController frontController;

    /** The password encoder. */
    private final PasswordEncoder passwordEncoder = new Md5PasswordEncoder();

    /** The user session. */
    protected UserSession userSession;

    /** The user state service. */
    protected UserStateService userStateService = new FakeUserStateService();

    /** The view factory service. */
    protected ViewFactoryService viewFactoryService = context
            .mock(ViewFactoryService.class);

    /**
     * Creates the initial user session.
     *
     * @return the user session
     */
    protected final UserSession createInitialUserSession() {
        ApplicationRequest applicationRequest = TestRequestFactory
                .createApplicationRequestSwedishUser();
        UserSession userSession = this.applicationManager.service(
                applicationRequest).getUserSessionDTO().getUserSession();
        userStateService.setUserSessionId(userSession.getId());
        return userSession;
    }

    /**
     * Prepare logged in as user.
     *
     * @param username the username
     * @param password the password
     * @throws Exception the exception
     */
    protected final void prepareLoggedInAsUser(final String username,final String password) throws Exception {
        UserSession createInitialUserSession = createInitialUserSession();

        LoginRequest loginRequest = new LoginRequest(createInitialUserSession
                .getId(), UserAccountActionEvent.Operation.Process_login,username, passwordEncoder.encodePassword("password",
                null));

        userSession = this.applicationManager.service(loginRequest)
                .getUserSessionDTO().getUserSession();
        this.userStateService.setUserSessionId(userSession.getId());
    }
        
    /**
     * Replace view factory with mock.
     */
    @Before
    public final void replaceViewFactoryWithMock() {
        for (ActionHandler actionHandler : frontController
                .getActionHandlersMap().values()) {
            actionHandler.setViewFactoryService(viewFactoryService);
            actionHandler.setUserStateService(userStateService);
        }
        frontController.getDefaultActionHandler().setUserStateService(userStateService);
        frontController.getDefaultActionHandler().setViewFactoryService(viewFactoryService);        
    }

    /**
     * Setup database pre condition.
     *
     * @throws Exception the exception
     */
    @BeforeTransaction
    public abstract void setupDatabasePreCondition() throws Exception;


    /**
     * Setup expected model and view.
     *
     * @throws Exception the exception
     */
    @Before
    public abstract void setupExpectedModelAndView() throws Exception;

    /**
     * Verify model and view.
     *
     * @throws Exception the exception
     */
    @After
    public abstract void verifyModelAndView() throws Exception;

    /**
     * Verify post condition.
     *
     * @throws Exception the exception
     */
    @AfterTransaction
    public final void verifyPostCondition() throws Exception {
        context.assertIsSatisfied();        
    }
}
